@@ -30,7 +30,25 @@ class DankEncoder:
         self.regex = DankEncoder.preprocess(regex)
         self.max_fixed_slice = fixed_slice
         self.fixed_slice = fixed_slice
-        self._encoder = DFAEncoder(self.regex, fixed_slice)
+        self._encoder = DFAEncoder()
+        self._encoder.from_regex(regex, fixed_slice)
+
+    @staticmethod
+    def from_fst(fst: str, fixed_slice: int):
+        ret = DankEncoder("(a|b)", fixed_slice)
+        ret._encoder = DFAEncoder()
+        ret._encoder.from_fst(fst, fixed_slice)
+        return ret
+
+    def num_states(self) -> int:
+        """Returns the number of states in the underlying DFA representation of
+        the provided regex. The number of states can be useful in order to infer
+        the fixed slice value in certain instances.
+
+        :return: the number of states found within the underlying DFA 
+        :rtype: int
+        """
+        return self._encoder.num_states()
 
     def num_words(self, low: int, high: int) -> int:
         """Returns the number of elements of the regular language given an upper
@@ -46,8 +64,8 @@ class DankEncoder:
         :return: the number of elements bounded by [low, high] in the language
         :rtype: int
         """
-        assert low > 0 and low < high
-        assert high > 0 and high < self.max_fixed_slice
+        assert low > 0 and low <= high
+        assert high > 0 and high <= self.max_fixed_slice
         return self._encoder.num_words(low, high)
 
     def set_fixed_slice(self, n: int):
@@ -130,7 +148,7 @@ class DankEncoder:
         :rtype: str
         """
         # First pass, expand all (simple) character classes
-        char_classes = re.findall(r'\[(.*?)\]', regex)
+        char_classes = re.findall(r'\[(.\-.)\]', regex)
         for cc in char_classes:
             if re.match(r'^.\-.$', cc):
                 regex = regex.replace(f'[{cc}]', DankEncoder.range_expand(cc))
@@ -145,7 +163,10 @@ class DankEncoder:
         parse_stack, expr_stack, rep_stack = [], [], []
         while i < len(regex):
             e = regex[i]
-            if e == '(':
+            # Skip parsing the next character if it has been escaped
+            if e == '\\':
+                i += 1
+            elif e == '(':
                 parse_stack.append(i)
             elif e == ')':
                 start = parse_stack.pop()
